export declare type ChangeTypes = "added" | "modified" | "deleted";
declare type SubscribeOptions = Partial<{
    existing: boolean;
}>;
declare type SubscriptionCallback<T> = (change: {
    type: ChangeTypes;
    doc: {
        ref: IDocumentRef<T>;
        data: T;
    };
}) => void;
export interface ICollectionRef<U = any> {
    readonly name: string;
    doc(id?: string): IDocumentRef;
    keys(): Promise<string[]>;
    push<T = U>(data: T): Promise<IDocumentRef>;
    subscribe<T = U>(types: ChangeTypes[], options: SubscribeOptions, onData: SubscriptionCallback<T>): Promise<{
        unsubscribe: () => void;
    }>;
    /**
     * Only if root key available
     */
    delete(): Promise<void>;
}
export interface IDocumentRef<T = any> {
    readonly key: string;
    get(): Promise<T>;
    set(data: T): Promise<void>;
    update(update: never): Promise<void>;
    delete(): Promise<void>;
    subscribe(types: ChangeTypes[], options: SubscribeOptions, onData: SubscriptionCallback<T>): Promise<{
        unsubscribe: () => void;
    }>;
    collection(name: string): ICollectionRef;
}
export default class Database {
    static WebSocket: typeof WebSocket;
    private initLock;
    private socket;
    private requests;
    private eventhandler;
    constructor(url: string, database: string, accesskey?: string, authkey?: string, rootkey?: string);
    private handleResponse;
    private handleEvent;
    private queryRequest;
    private subscribe;
    private static CollectionRef;
    private static DocumentRef;
    collection(name: string): ICollectionRef;
    collections(): Promise<ICollectionRef[]>;
    close(): void;
}
export {};
