"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// import * as io from "socket.io-client";
const WebSocket = require("ws");
const utils_1 = require("@hibas123/utils");
const ALPHABET = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
const ID_LENGTH = 8;
const generate = () => {
    var rtn = '';
    for (var i = 0; i < ID_LENGTH; i++) {
        rtn += ALPHABET.charAt(Math.floor(Math.random() * ALPHABET.length));
    }
    return rtn;
};
class Database {
    constructor(url, database, accesskey, authkey, rootkey) {
        this.initLock = new utils_1.Lock();
        this.requests = new Map();
        this.eventhandler = new Map();
        const u = new URL(url);
        if (authkey)
            u.searchParams.set("authkey", authkey);
        if (rootkey)
            u.searchParams.set("rootkey", rootkey);
        if (accesskey)
            u.searchParams.set("accesskey", accesskey);
        u.searchParams.set("database", database);
        console.log("URL:", u.href);
        this.initLock.getLock().then(lock => {
            this.socket = new WebSocket(u.href);
            this.socket.on("event", this.handleEvent.bind(this));
            this.socket.onopen = () => lock.release();
            this.socket.onerror = err => { console.error(err); lock.release(); };
            this.socket.onmessage = msg => {
                // console.log("Message:", msg)
                let data = JSON.parse(msg.data);
                switch (data.ns) {
                    case "message":
                        this.handleResponse(...data.args);
                        break;
                    case "event":
                        this.handleEvent(...data.args);
                        break;
                    case "error_msg":
                        // TODO: Some kind of error handling etc.
                        console.error("Server error:", data.args[0]);
                        break;
                }
            };
        });
    }
    handleResponse(id, err, data) {
        let req = this.requests.get(id);
        if (req) {
            if (err)
                req(new Error(data), undefined);
            else
                req(undefined, data);
        }
        else {
            console.error("Invalid response", id);
        }
    }
    handleEvent(id, data) {
        let eh = this.eventhandler.get(id);
        if (eh)
            eh(data);
    }
    async queryRequest(type, path, data, id) {
        await this.initLock.getLock().then(l => l.release());
        if (typeof path === "string")
            path = path.split("/").filter(e => e !== "");
        if (!id)
            id = generate();
        return new Promise((yes, no) => {
            this.requests.set(id, (err, data) => {
                err ? no(err) : yes(data);
                this.requests.delete(id);
            });
            this.socket.send(JSON.stringify({ ns: "query", args: [id, type, path, data] }));
        });
    }
    get(path) {
        return this.queryRequest("get", path);
    }
    set(path, data) {
        return this.queryRequest("set", path, data);
    }
    push(path, data) {
        return this.queryRequest("push", path, data);
    }
    async subscribe(path, type, ondata) {
        const id = generate();
        const subscriptionID = await this.queryRequest("subscribe", path, type, id);
        this.eventhandler.set(subscriptionID, ondata);
        return {
            unsubscribe: () => {
                this.queryRequest("unsubscribe", path, undefined, id);
                this.eventhandler.delete(subscriptionID);
            }
        };
    }
}
exports.default = Database;
//# sourceMappingURL=index.js.map